#!/bin/sh
# simple shell script to build an image of this system onto flash.  This
# image is as minimal as possible.

#
# This script assumes that you have defined MAKEOBJDIRPREFIX to point to
#  the location that contains your object files from a clean FreeBSD
#  'make buildworld'.
# It also assumes that you have defined FreeBSDSrcDir as the directory
#  that is the top of your FreeBSD source tree.
#
# There are also some hand tweaks for the different kinds of machines we
# make.  This is a mess and should be cleaned up.
#

#
# env vars used:
#  DevSize            - size of /dev mfs partition (sets 'devsize' parameter
#                         in target's /etc/rc.conf)
#  DiskConfig         - name of the disk config file used to invoke 'diskprep'
#  ExtraFileDirs      - List of directories which should be copied recursive to
#			/ of the flash.
#  FreeBSDProgramDirs - list of dirs under FreeBSDSrcDir of stuff that
#                         should be installed
#  FreeBSDSrcDir      - where the top level Makefile.inc1 resides for the
#                         FreeBSD source tree (like /usr/src)
#  HasSerialConsole   - put '-h' in target's /boot.config if YES
#  KernelName         - name of the kernel config file (used to find the
#                         directory containing the desired kernel)
#  MAKEOBJDIRPREFIX   - where FreeBSD was built (like /usr/obj)
#  PasswordEntry      - an optional password file entry to go into
#                         /etc/master.passwd on the target
#  PruneList          - List of files freebsd installs that we should ax.
#  RootDiskUnit       - the target's root disk unit number (written to
#                         the target's /boot/loader.conf)
#  TargetDevice       - the device name (without the /dev) used to find
#                         the target when building.  This is not the same
#                         as the device name used by the disk after installed
#                         in its final destination.  It is the name
#                         of the device when the device is on the build
#                         system.
#  TargetRootDevice   - name of the device as used on the destination
#                         system, not the build system.
#  TargetTmpMount     - temporary mount point
#  VarSize            - size of /var mfs partition (sets 'varsize' parameter
#                         in target's /etc/rc.conf)
#  
#

die()
{
    echo mkflash: $*
    exit 1
}

mkdiskdevs()
{
    (cd $2/dev ; sh MAKEDEV $1 ${1}s${3}a)
}

populatedisk()
{
    (cd $FreeBSDSrcDir
    make -f Makefile.inc1 hierarchy DESTDIR=$1 NOMAN=yes
    (cd etc ; make distribution DESTDIR=$1 NOMAN=yes)
    for i in ${FreeBSDProgramDirs}; do
        echo "==> $i"
        test -d $i && (cd $i ; make install -DNOINFO -DNOMAN DESTDIR=$1 -DNOPROFILE)
    done)
    rm $1/usr/lib/*.a
    mkdir $1/mod
    mkdir $1/altmod

    echo "# device	mnt	type	type	dump   	pass" > $1/etc/fstab
    echo "/dev/$3s1a	/	ufs	ro	1 1" \
	>> $1/etc/fstab
    echo "/dev/$3s1e	/mod	ufs	rw,noatime 1 1" \
	>> $1/etc/fstab
    echo "/dev/$3s1f	/altmod	ufs	rw,noatime,noauto 1 1" \
	>> $1/etc/fstab
}

mkrcconf()
{
    echo '# rc.conf generated from mkflash' > $1
    echo 'sendmail_enable=NO' >> $1
    echo 'portmap_enable=NO' >> $1
    echo 'network_interfaces="auto"' >> $1
    echo 'ifconfig_ed0="DHCP"' >> $1
    echo 'ifconfig_ed1="DHCP"' >> $1
    echo 'ifconfig_rl0="DHCP"' >> $1
    echo 'ifconfig_fxp0="DHCP"' >> $1
    echo 'hostname=compactFlash' >> $1
    echo 'syslogd_enable=NO' >> $1
    echo 'cron_enable=NO' >> $1
    echo 'root_rw_mount=NO' >> $1
    echo 'diskless_mount=/etc/rc.diskless2' >> $1
    echo "varsize=${VarSize}" >> $1
    echo "devsize=${DevSize}" >> $1
    echo 'update_motd=NO' >> $1
}

installkernel()
{
    (cd ${FreeBSDSrcDir}
    if [ ! -d sys/compile/$1 ] ; then
	die "Prebuilt kernel $1 doesn't actually exist, punting"
    fi
    cd sys/compile/$1
    echo "===> Installing kernel $1"
    make install DESTDIR=${TargetTmpMount})
}

fixttys()
{
    mv ${TargetTmpMount}/etc/ttys ${TargetTmpMount}/etc/ttys-
    sed -e '/^ttyd0/s/off/on /' < ${TargetTmpMount}/etc/ttys- \
	> ${TargetTmpMount}/etc/ttys
    rm ${TargetTmpMount}/etc/ttys-
}

fixpam()
{
    mv ${TargetTmpMount}/etc/pam.conf ${TargetTmpMount}/etc/pam.conf-
    sed -e '/pam_skey.so/s/^/#/' -e '/pam_clear/s/^/#/' \
	< ${TargetTmpMount}/etc/pam.conf- > ${TargetTmpMount}/etc/pam.conf
    rm ${TargetTmpMount}/etc/pam.conf-
}

mountdisk()
{
    mount /dev/$1s1a /mnt
}

copyextra()
{
    for i; do
	if [ -d $i ]; then
            (cd $i ; tar -cf - --exclude CVS . | (cd ${TargetTmpMount} ; tar xvf -))
	fi
    done
}

prunelist()
{
    for i; do
        rm ${TargetTmpMount}/$i
    done
}

main()
{
    mkdiskdevs ${TargetDevice} "" 1
    umount ${TargetTmpMount}
    diskprep --config ${DiskConfig} ${TargetDevice}
    mountdisk ${TargetDevice}
    populatedisk ${TargetTmpMount} ${TargetDevice} ${TargetRootDevice:=${TargetDevice}}
    mkdiskdevs ${TargetRootDevice:=${TargetDevice}} ${TargetTmpMount} 1
    mkrcconf ${TargetTmpMount}/etc/rc.conf
    rm -rf ${TargetTmpMount}/var
    mkdir ${TargetTmpMount}/var
    chmod 755 ${TargetTmpMount}/var
    chown root:wheel ${TargetTmpMount}/var
    rm -rf ${TargetTmpMount}/tmp
    ln -s /var/tmp ${TargetTmpMount}/tmp
    copyextra ${ExtraFileDirs}
    if [ "${HasSerialConsole}" = "YES" ]; then
	echo -h > ${TargetTmpMount}/boot.config
    fi
    fixttys
    fixpam
    echo "# Do not edit, generated by mkflash" >  ${TargetTmpMount}/boot/loader.conf
    if [ ! -z "${RootDiskUnit}" ]; then
	echo "root_disk_unit=${RootDiskUnit}" >> ${TargetTmpMount}/boot/loader.conf
    fi
    installkernel ${KernelName}
    if [ ! -z "${PasswordEntry}" ]; then
	echo "${PasswordEntry}" >> ${TargetTmpMount}/etc/master.passwd
	pwd_mkdb -p -d ${TargetTmpMount}/etc ${TargetTmpMount}/etc/master.passwd
	chown -R 1001.1001 ${TargetTmpMount}/usr/tsc
    fi
    prunelist ${PruneList}
    df ${TargetTmpMount}
    umount ${TargetTmpMount}
    tunefs -n disable /dev/r${TargetDevice}s1a
}

if [ -z "$1" ]; then
    echo "Must tell me the name of the config file to use."
fi

for i; do
    . $i
done

if [ X"$MAKEOBJDIRPREFIX" = X ]; then
    echo "No MAKEOBJDIRPREFIX, you lose"
    exit 1
fi


if [ ! -f "${FreeBSDSrcDir}/Makefile.inc1" ]; then
    echo "Makefile.inc1 not found"
    exit 1
fi

main
